#include <iterator>
#include "ball.hpp"
#include "defs.hpp"


void
ball_t::init(const vector_type& v)
{
	o = p = v;
	direction = normalize(vector_type(1, -1));
	speed = 1;
	for(int i = 0; i < orbit_size; ++i)
		M_orbit[i].clear();
}

void
ball_t::update()
{
	o = p;
	p += direction * speed;

	for(int i = 0; i < orbit_size - 1; ++i)
		M_orbit[i].assign(M_orbit[i + 1].begin(), M_orbit[i + 1].end());
	M_orbit[orbit_size - 1].clear();
}

void
ball_t::draw(oiran::linda::graphics_t& g)
{
	orbit_list_type orbit_list;

	for(int i = 0; i < orbit_size; ++i)
		if(!M_orbit[i].empty())
			orbit_list.insert(orbit_list.end(),
					M_orbit[i].begin(), M_orbit[i].end());

	g.color(front_color, 0.5);
	orbit_list_type::const_iterator i = orbit_list.begin();
	for(; i != orbit_list.end(); ++i)
	{
		orbit_list_type::const_iterator j = i;
		std::advance(j, 1);
		if(j == orbit_list.end())
			break;

		g.line(i->x, i->y, j->x, j->y);
	}
	g.point(p.x, p.y);
}

void
ball_t::push_orbit(const vector_type& v)
{
	M_orbit[orbit_size - 1].push_back(v);
}
