#include "resource.hpp"
#include "scene.hpp"


namespace {


	const int offset_y = (height - field_height * block_size) / 2;
	const int offset_x = block_size * 2 + field_width * block_size + block_size;
	const int message_delay = framerate * 3 / 2;


} // unnamed namespace.


scene_edit_t::scene_edit_t(int stage) :
		M_field(stage),
		M_select(0),
		M_x(0),
		M_y(0),
		M_effect_count(0)
{
	M_field.load();
}

void
scene_edit_t::update()
{
	oiran::linda::mouse_t& mouse = resource_t::instance()->mouse;

	M_x = mouse.x();
	M_y = mouse.y();

	if(M_effect_count > 0)
		--M_effect_count;

	if(block_size * 2 <= M_x &&
			M_x < block_size * 2 + field_width * block_size &&
			offset_y <= M_y &&
			M_y < offset_y + field_height * block_size)
	{
		int index_x = (M_x - block_size * 2) / block_size;
		int index_y = (M_y - offset_y) / block_size;

		if(M_select && mouse.is_pressed(mouse.left))
			M_field.put(M_select, index_x, index_y);
		else if(mouse.is_pressed(mouse.right))
			M_field.put(0, index_x, index_y);
	}

	if(mouse.is_pushed(mouse.left))
	{
		if(block_size * 2 > M_x ||
				M_x >= block_size * 2 + field_width * block_size ||
				offset_y > M_y ||
				M_y >= offset_y + field_height * block_size)
		{
			M_select = 0;

			for(int i = 0; i < number_of_blocks; ++i)
				if(offset_x <= mouse.x() && mouse.x() <= offset_x + block_size &&
						offset_y + i * block_size * 2 <= mouse.y() &&
						mouse.y() <= offset_y + (i + 1) * block_size * 2)
					M_select = static_cast<char>(wall + i);

			if(offset_x <= M_x && M_x <= offset_x + 4 * block_size &&
					height - offset_y - block_size <= M_y &&
					M_y <= height - offset_y)
				valid(false);

			if(offset_x <= M_x && M_x <= offset_x + 4 * block_size &&
					height - offset_y - block_size * 3 <= M_y &&
					M_y <= height - offset_y - block_size * 2)
			{
				M_field.save();
				M_effect_count = message_delay;
			}
		}
	}
}

void
scene_edit_t::draw(oiran::linda::graphics_t& g)
{
	oiran::linda::image_t* images = resource_t::instance()->images;
	oiran::linda::simple_font_t& font = resource_t::instance()->font;

	g.push_matrix();
	g.translate(block_size * 2, offset_y);

	g.color(front_color);
	M_field.draw(g);

	g.stroke();
	g.rect(0, 0, field_width * block_size, field_height * block_size);
	g.fill();

	g.pop_matrix();

	const char* const block_name_list[] = {
		"wall",
		"normal",
	};

	font.size(block_size, block_size);
	for(int i = 0; i < number_of_blocks; ++i)
	{
		images[i].draw(offset_x, offset_y + i * block_size * 2);
		font.print(offset_x + block_size * 1.5, offset_y + i * block_size * 2,
				block_name_list[i]);
	}

	font.print(offset_x, height - offset_y - block_size, "EXIT");
	font.print(offset_x, height - offset_y - block_size * 3, "SAVE");

	if(M_select)
		images[M_select - 1].draw(M_x - block_size / 2, M_y - block_size / 2);

	if(M_effect_count)
	{
		g.color(sub_color);
		font.print(block_size * 2, height - offset_y + block_size / 2, "SAVED.");
	}
}
