#include <sstream>
#include <iomanip>
#include "scene.hpp"
#include "resource.hpp"


namespace {


	const int thumbnail_width = field_width * block_size / 4;
	const int thumbnail_height = field_height * block_size / 4;
	const int space_width = (width - thumbnail_width * 3) / 4;
	const int space_height = (height - thumbnail_height * 3) / 4;

	const int offset_x[] = {
		space_width * 3 / 2,
		space_width * 2 + thumbnail_width,
		space_width * 5 / 2 + thumbnail_width * 2,
	};
	const int offset_y[] = {
		space_height * 3 / 2,
		space_height * 2 + thumbnail_height,
		space_height * 5 / 2 + thumbnail_height * 2,
	};

	const int left_x = offset_x[0] - space_width - block_size;
	const int left_y = (height - block_size * 4) / 2;
	const int right_x = offset_x[2] + thumbnail_width + space_width;
	const int right_y = (height - block_size * 4) / 2;


} // unnamed namespace.


scene_stage_list_t::scene_stage_list_t(scene_manager_t* manager, int index) :
		M_manager(manager),
		M_index(index),
		M_select(-1),
		M_left(false),
		M_right(false)
{
	std::fill(M_fields, M_fields + 9, reinterpret_cast<field_t*>(0));
	_reload();
}

scene_stage_list_t::~scene_stage_list_t()
{
	_clear();
}

void
scene_stage_list_t::update()
{
	oiran::linda::mouse_t& mouse = resource_t::instance()->mouse;

	M_select = -1;
	M_left = false;
	M_right = false;

	for(int y = 0; y < 3; ++y)
		for(int x = 0; x < 3; ++x)
			if(offset_x[x] <= mouse.x() &&
					mouse.x() <= offset_x[x] + thumbnail_width &&
					offset_y[y] <= mouse.y() &&
					mouse.y() <= offset_y[y] + thumbnail_height)
				M_select = y * 3 + x;

	if(left_x <= mouse.x() && mouse.x() <= left_x + block_size &&
			left_y <= mouse.y() && mouse.y() <= left_y + block_size * 4)
		M_left = true;
	if(right_x <= mouse.x() && mouse.x() <= right_x + block_size &&
			right_y <= mouse.y() && mouse.y() <= right_y + block_size * 4)
		M_right = true;

	if(M_select != -1)
	{
		if(mouse.is_pushed(mouse.left))
		{
			if(!M_fields[M_index + M_select] ||
					!M_fields[M_index + M_select]->valid())
			{
				M_manager->push(scene_stage_list + M_index);
				M_manager->push(scene_edit + M_index + M_select);
				valid(false);
			}
			else
			{
				M_manager->push(scene_stage_list + M_index);
				M_manager->push(scene_stage + M_index + M_select);
				valid(false);
			}
		}
		else if(mouse.is_pushed(mouse.right))
		{
			M_manager->push(scene_stage_list + M_index);
			M_manager->push(scene_edit + M_index + M_select);
			valid(false);
		}
	}

	if(M_index > 0 && M_left && mouse.is_pushed(mouse.left))
	{
		M_index -= 9;
		_reload();
	}
	if(M_index < max_stage - 9 && M_right && mouse.is_pushed(mouse.left))
	{
		M_index += 9;
		_reload();
	}
}

void
scene_stage_list_t::draw(oiran::linda::graphics_t& g)
{
	oiran::linda::simple_font_t& font = resource_t::instance()->font;

	g.color(0xffffff);
	font.size(48, 48);
	font.print(1, 1, "BB");

	font.size(12, 24);
	for(int y = 0; y < 3; ++y)
		for(int x = 0; x < 3; ++x)
		{
			g.color(0xffffff, 0.5);

			g.push_matrix();
			g.translate(offset_x[x], offset_y[y]);
			g.scale(0.25);
			field_t* field = M_fields[y * 3 + x];
			if(field)
				field->draw(g);

			if(M_select == y * 3 + x)
				g.color(0xffffff);

			g.stroke();
			g.rect(0, 0, field_width * block_size, field_height * block_size);
			g.fill();
			g.pop_matrix();

			std::ostringstream oss;
			oss << std::setfill('0') << std::setw(2) << M_index + y * 3 + x;

			font.print(offset_x[x] + (thumbnail_width - 24) / 2,
					offset_y[y] + (thumbnail_height - 24) / 2, oss.str());
		}

	if(M_index > 0)
	{
		if(M_left)
			g.color(0xffffff);
		else
			g.color(0xffffff, 0.5);

		g.triangle(
				left_x, height / 2,
				left_x + block_size, left_y,
				left_x + block_size, left_y + block_size * 4);
	}
	if(M_index < max_stage - 9)
	{
		if(M_right)
			g.color(0xffffff);
		else
			g.color(0xffffff, 0.5);

		g.triangle(
				right_x + block_size, height / 2,
				right_x, right_y,
				right_x, right_y + block_size * 4);
	}
}

void
scene_stage_list_t::_clear()
{
	for(int i = 0; i < 9; ++i)
		if(M_fields[i])
		{
			delete M_fields[i];
			M_fields[i] = 0;
		}
}

void
scene_stage_list_t::_reload()
{
	_clear();
	for(int i = 0; i < 9; ++i)
	{
		M_fields[i] = new field_t(M_index + i);
		M_fields[i]->load();
	}
}
