#ifndef OIRAN_FLYWEIGHT_HPP
#define OIRAN_FLYWEIGHT_HPP

#include <map>
#include <oiran/singleton.hpp>

namespace oiran {


	// flyweight pattern.
	template <typename Key, typename Value>
	class flyweight_t : public singleton_t<flyweight_t<Key, Value> >
	{
	public:
		typedef std::map<Key, Value*> cache_type;

		~flyweight_t() { clear(); }

		// get a pointer to an object with key.
		Value* get(const Key& key)
		{
			typedef typename cache_type::iterator iterator;
			typedef typename cache_type::value_type value_type;

			iterator i = M_cache.find(key);
			if(i == M_cache.end())
			{
				std::pair<iterator, bool> result =
						M_cache.insert(value_type(key, new Value(key)));
				// no check result.second.
				i = result.first;
			}
			return i->second;
		}

		void clear()
		{
			typedef typename cache_type::iterator iterator;

			for(iterator i = M_cache.begin(); i != M_cache.end(); ++i)
				delete i->second;
			M_cache.clear();
		}

	private:
		cache_type M_cache;
	};


} // namespace oiran.

#endif // OIRAN_FLYWEIGHT_HPP
