#include <SDL.h>
#ifdef USE_SDL_IMAGE
#include <SDL_image.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <assert.h>
#include <stdexcept>
#include "image.hpp"
#include "gl.hpp"
#ifndef NDEBUG
#include <iostream>
#endif

namespace oiran { namespace linda {


	namespace {


		SDL_Surface*
		load_32bits_image(const char* file_name)
		{
#ifdef USE_SDL_IMAGE
			SDL_Surface* image = IMG_Load(file_name);
			if(!image)
				throw std::runtime_error(IMG_GetError());
#else
			SDL_Surface* image = SDL_LoadBMP(file_name);
			if(!image)
				throw std::runtime_error(SDL_GetError());
#endif

			SDL_PixelFormat format;
			format.palette = 0;
			format.BitsPerPixel = 32;
			format.BytesPerPixel = 4;
			format.Rloss = format.Gloss = format.Bloss = format.Aloss = 0;
			format.Rshift = 0;
			format.Gshift = 8;
			format.Bshift = 16;
			format.Ashift = 24;
			format.Rmask = 0x000000ff;
			format.Gmask = 0x0000ff00;
			format.Bmask = 0x00ff0000;
			format.Amask = 0xff000000;
			format.colorkey = 0;
			format.alpha = 0;

			SDL_Surface* result =
					SDL_ConvertSurface(image, &format, SDL_SWSURFACE);
			SDL_FreeSurface(image);
			if(!result)
				throw std::runtime_error(SDL_GetError());

			return result;
		}


	} // unnamed namespace.



	image_t::image_t() :
			M_width(0),
			M_height(0),
			M_id(0)
	{
	}

	image_t::~image_t()
	{
		if(M_id)
			glDeleteTextures(1, &M_id);
	}

	void
	image_t::load(const char* file_name)
	{
		assert(M_id == 0 && "image is already loaded.");

		SDL_Surface* image = load_32bits_image(file_name);

		glGenTextures(1, &M_id);
		glBindTexture(GL_TEXTURE_2D, M_id);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 4, image->w, image->h,
				GL_RGBA, GL_UNSIGNED_BYTE, image->pixels);
		glTexParameteri(GL_TEXTURE_2D,
				GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glBindTexture(GL_TEXTURE_2D, 0);
		SDL_FreeSurface(image);

		check_gl_error();

		M_width = image->w;
		M_height = image->h;

#ifndef NDEBUG
		std::cout << "loaded image : " << file_name << std::endl;
#endif
	}

	void
	image_t::draw(double x, double y)
	{
		if(!M_id)
			return;

		glBindTexture(GL_TEXTURE_2D, M_id);
		glBegin(GL_QUADS);
		glTexCoord2d(0, 0); glVertex2d(x, y);
		glTexCoord2d(0, 1); glVertex2d(x, y + M_height);
		glTexCoord2d(1, 1); glVertex2d(x + M_width, y + M_height);
		glTexCoord2d(1, 0); glVertex2d(x + M_width, y);
		glEnd();
		glBindTexture(GL_TEXTURE_2D, 0);
	}


}} // namespace oiran::linda.
