#ifndef OIRAN_LINDA_SCENE_HPP
#define OIRAN_LINDA_SCENE_HPP

#include <memory>
#include <stack>
#include <oiran/noncopyable.hpp>
#include <oiran/linda/graphics.hpp>

namespace oiran { namespace linda {


	class scene_t
	{
	public:
		scene_t();
		virtual ~scene_t();

		virtual void update() = 0;
		virtual void draw(graphics_t& g) = 0;

		void valid(bool flag) { M_valid = flag; }
		bool valid() const { return M_valid; }

	private:
		bool M_valid;
	};


	class scene_manager_t
	{
	public:
		scene_manager_t();
		virtual ~scene_manager_t();

		void push(int id);
		void update();
		void draw(graphics_t& g);

	protected:
		virtual scene_t* create(int id) = 0;

	private:
		OIRAN_NONCOPYABLE(scene_manager_t);

	private:
		std::auto_ptr<scene_t> M_active;
		std::stack<int> M_scene_ids;
	};


}} // namespace oiran::linda.

#endif // OIRAN_LINDA_SCENE_HPP
